/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package neuralnetworkbase

import (
	mat "gonum.org/v1/gonum/mat"
)

const (
	BiasGradient   = iota
	WeightGradient = iota
)

type GradientDescentInitializer func(nn *NeuralNetwork, layer, gradientType int) interface{}

type OnlineGradientDescent interface {
	ApplyDelta(m mat.Matrix, gradient mat.Matrix) *mat.Dense
}

type BatchGradientDescent interface {
	ApplyDelta(m mat.Matrix) *mat.Dense
	AccumGradients(gradient mat.Matrix)
	Gradients() *mat.Dense
}

const (
	StateIdle       = 1
	StateLearning   = 2
	StateValidation = 3
	StatePredict    = 4
)

type StateWatcher interface {
	Init(nn *NeuralNetwork)
	UpdateState(state int)
	UpdateActivations(l int, a *mat.Dense)
	UpdateBiases(l int, biases *mat.Dense)
	UpdateWeights(l int, weights *mat.Dense)
}
