/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package neuralnetworkbase

import (
	"math"
	rand "math/rand"
	"time"

	mat "gonum.org/v1/gonum/mat"
)

func generateRandomDense(rows, columns int) *mat.Dense {
	rand.Seed(time.Now().UnixNano())
	data := make([]float64, rows*columns)
	for i := range data {
		data[i] = rand.NormFloat64()
	}
	return mat.NewDense(rows, columns, data)
}

func applySigmoid(_, _ int, x float64) float64 {
	return sigmoid(x)
}

func applySigmoidPrime(_, _ int, x float64) float64 {
	return sigmoidPrime(x)
}

func sigmoid(x float64) float64 {
	return 1.0 / (1.0 + math.Exp(-x))
}

func sigmoidPrime(x float64) float64 {
	sig := sigmoid(x)
	return sig * (1 - sig)
}

func sign(v float64) float64 {
	if v == 0 {
		return 0
	}
	// fmt.Printf("%v / math.Abs(%v) = %v\n", v, math.Abs(v), v/math.Abs(v))
	return v / math.Abs(v)
}
